@@ -1,20 +1,20 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2021 The HyperSpy developers
+# Copyright 2007-2022 The HyperSpy developers
 #
-# This file is part of  HyperSpy.
+# This file is part of HyperSpy.
 #
-#  HyperSpy is free software: you can redistribute it and/or modify
+# HyperSpy is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#  HyperSpy is distributed in the hope that it will be useful,
+# HyperSpy is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
+# along with HyperSpy. If not, see <https://www.gnu.org/licenses/#GPL>.
 
 import numpy as np
 from scipy import constants
@@ -25,7 +25,7 @@ from hyperspy._signals.complex_signal1d import (ComplexSignal1D,
 from hyperspy.misc.eels.tools import eels_constant
 
 
-class DielectricFunction_mixin:
+class DielectricFunction(ComplexSignal1D):
 
     _signal_type = "DielectricFunction"
     _alias_signal_types = ["dielectric function"]
@@ -112,6 +112,10 @@ class DielectricFunction_mixin:
         return neff1, neff2
 
     def get_electron_energy_loss_spectrum(self, zlp, t):
+        for axis in self.axes_manager.signal_axes:
+            if not axis.is_uniform:
+                raise NotImplementedError(
+                    "The function is not implemented for non-uniform axes.")
         data = ((-1 / self.data).imag * eels_constant(self, zlp, t).data *
                 self.axes_manager.signal_axes[0].scale)
         s = self._deepcopy_with_new_data(data)
@@ -122,9 +126,5 @@ class DielectricFunction_mixin:
         return s
 
 
-class DielectricFunction(DielectricFunction_mixin, ComplexSignal1D):
-    pass
-
-
 class LazyDielectricFunction(DielectricFunction, LazyComplexSignal1D):
     pass
