@@ -1,20 +1,20 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2021 The HyperSpy developers
+# Copyright 2007-2022 The HyperSpy developers
 #
-# This file is part of  HyperSpy.
+# This file is part of HyperSpy.
 #
-#  HyperSpy is free software: you can redistribute it and/or modify
+# HyperSpy is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#  HyperSpy is distributed in the hope that it will be useful,
+# HyperSpy is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
+# along with HyperSpy. If not, see <https://www.gnu.org/licenses/#GPL>.
 
 import numpy as np
 from dask.array import Array as dArray
@@ -22,11 +22,11 @@ import traits.api as t
 from traits.trait_numeric import Array
 import sympy
 from sympy.utilities.lambdify import lambdify
-from distutils.version import LooseVersion
+from packaging.version import Version
 from pathlib import Path
 
 import hyperspy
-from hyperspy.misc.utils import slugify
+from hyperspy.misc.utils import slugify, is_binned
 from hyperspy.misc.io.tools import (incremental_filename,
                                     append2pathname,)
 from hyperspy.misc.export_dictionary import export_to_dictionary, \
@@ -112,7 +112,7 @@ class Parameter(t.HasTraits):
     """
     __number_of_elements = 1
     __value = 0
-    __free = True
+    _free = True
     _bounds = (None, None)
     __twin = None
     _axes_manager = None
@@ -157,6 +157,7 @@ class Parameter(t.HasTraits):
         self.grad = None
         self.name = ''
         self.units = ''
+        self._linear = False
         self.map = None
         self.model = None
         self._whitelist = {'_id_name': None,
@@ -168,6 +169,7 @@ class Parameter(t.HasTraits):
                            '_bounds': None,
                            'ext_bounded': None,
                            'name': None,
+                           '_linear':None,
                            'ext_force_positive': None,
                            'twin_function_expr': None,
                            'twin_inverse_function_expr': None,
@@ -358,16 +360,19 @@ class Parameter(t.HasTraits):
     # Fix the parameter when coupled
     def _get_free(self):
         if self.twin is None:
-            return self.__free
+            return self._free
         else:
             return False
 
     def _set_free(self, arg):
-        old_value = self.__free
-        self.__free = arg
+        if arg and self.twin:
+            raise ValueError(f"Parameter {self.name} can't be set free "
+                             "is twinned with {self.twin}.")
+        old_value = self._free
+        self._free = arg
         if self.component is not None:
             self.component._update_free_parameters()
-        self.trait_property_changed('free', old_value, self.__free)
+        self.trait_property_changed('free', old_value, self._free)
 
     def _on_twin_update(self, value, twin=None):
         if (twin is not None
@@ -732,7 +737,7 @@ class Component(t.HasTraits):
     active = t.Property(t.CBool(True))
     name = t.Property(t.Str(''))
 
-    def __init__(self, parameter_name_list):
+    def __init__(self, parameter_name_list, linear_parameter_list=None):
         self.events = Events()
         self.events.active_changed = Event("""
             Event that triggers when the `Component.active` changes.
@@ -748,7 +753,7 @@ class Component(t.HasTraits):
                 The new active state
             """, arguments=["obj", 'active'])
         self.parameters = []
-        self.init_parameters(parameter_name_list)
+        self.init_parameters(parameter_name_list, linear_parameter_list)
         self._update_free_parameters()
         self.active = True
         self._active_array = None # only if active_is_multidimensional is True
@@ -863,11 +868,31 @@ class Component(t.HasTraits):
         self.events.active_changed.trigger(active=self._active, obj=self)
         self.trait_property_changed('active', old_value, self._active)
 
-    def init_parameters(self, parameter_name_list):
+    def init_parameters(self, parameter_name_list, linear_parameter_list=None):
+        """
+        Initialise the parameters of the component.
+
+        Parameters
+        ----------
+        parameter_name_list : list
+            The list of parameter names.
+        linear_parameter_list : list, optional
+            The list of linear parameter. The default is None.
+
+        Returns
+        -------
+        None.
+
+        """
+        if linear_parameter_list is None:
+            linear_parameter_list = []
+
         for name in parameter_name_list:
             parameter = Parameter()
             self.parameters.append(parameter)
             parameter.name = name
+            if name in linear_parameter_list:
+                parameter._linear = True
             parameter._id_name = name
             setattr(self, name, parameter)
             if hasattr(self, 'grad_' + name):
@@ -941,8 +966,8 @@ class Component(t.HasTraits):
             shape = [1, ]
         if (not isinstance(self._active_array, np.ndarray)
                 or self._active_array.shape != shape):
-            _logger.debug('Creating _active_array for {}.\n\tCurrent array '
-                          'is:\n{}'.format(self, self._active_array))
+            _logger.debug(f'Creating _active_array for {self}.\n\tCurrent '
+                          f'array is:\n{self._active_array}')
             self._active_array = np.ones(shape, dtype=bool)
 
     def _create_arrays(self):
@@ -1068,7 +1093,7 @@ class Component(t.HasTraits):
             self.fetch_stored_values()
         return s
 
-    def set_parameters_free(self, parameter_name_list=None):
+    def set_parameters_free(self, parameter_name_list=None, only_linear=False, only_nonlinear=False):
         """
         Sets parameters in a component to free.
 
@@ -1078,12 +1103,17 @@ class Component(t.HasTraits):
             If None, will set all the parameters to free.
             If list of strings, will set all the parameters with the same name
             as the strings in parameter_name_list to free.
+        only_linear : bool
+            If True, only sets a parameter free if it is linear
+        only_nonlinear : bool
+            If True, only sets a parameter free if it is nonlinear
 
         Examples
         --------
         >>> v1 = hs.model.components1D.Voigt()
         >>> v1.set_parameters_free()
         >>> v1.set_parameters_free(parameter_name_list=['area','centre'])
+        >>> v1.set_parameters_free(linear=True)
 
         See also
         --------
@@ -1092,6 +1122,12 @@ class Component(t.HasTraits):
         hyperspy.model.BaseModel.set_parameters_not_free
         """
 
+        if only_linear and only_nonlinear:
+            raise ValueError(
+                "To set all parameters free, set both `only_linear` and "
+                "`only_nonlinear` to False."
+                )
+
         parameter_list = []
         if not parameter_name_list:
             parameter_list = self.parameters
@@ -1101,9 +1137,15 @@ class Component(t.HasTraits):
                     parameter_list.append(_parameter)
 
         for _parameter in parameter_list:
-            _parameter.free = True
+            if not only_linear and not only_nonlinear:
+                _parameter.free = True
+            elif only_linear and _parameter._linear:
+                _parameter.free = True
+            elif only_nonlinear and not _parameter._linear:
+                _parameter.free = True
 
-    def set_parameters_not_free(self, parameter_name_list=None):
+    def set_parameters_not_free(self, parameter_name_list=None,
+                                only_linear=False, only_nonlinear=False):
         """
         Sets parameters in a component to not free.
 
@@ -1113,12 +1155,17 @@ class Component(t.HasTraits):
             If None, will set all the parameters to not free.
             If list of strings, will set all the parameters with the same name
             as the strings in parameter_name_list to not free.
-
+        only_linear : bool
+            If True, only sets a parameter not free if it is linear
+        only_nonlinear : bool
+            If True, only sets a parameter not free if it is nonlinear
         Examples
         --------
         >>> v1 = hs.model.components1D.Voigt()
         >>> v1.set_parameters_not_free()
         >>> v1.set_parameters_not_free(parameter_name_list=['area','centre'])
+        >>> v1.set_parameters_not_free(only_linear=True)
+
 
         See also
         --------
@@ -1127,6 +1174,11 @@ class Component(t.HasTraits):
         hyperspy.model.BaseModel.set_parameters_not_free
         """
 
+        if only_linear and only_nonlinear:
+            raise ValueError(
+                "To set all parameters not free, "
+                "set both only_linear and _nonlinear to False.")
+
         parameter_list = []
         if not parameter_name_list:
             parameter_list = self.parameters
@@ -1136,18 +1188,23 @@ class Component(t.HasTraits):
                     parameter_list.append(_parameter)
 
         for _parameter in parameter_list:
-            _parameter.free = False
+            if not only_linear and not only_nonlinear:
+                _parameter.free = False
+            elif only_linear and _parameter._linear:
+                _parameter.free = False
+            elif only_nonlinear and not _parameter._linear:
+                _parameter.free = False
 
     def _estimate_parameters(self, signal):
-        self.binned = signal.metadata.Signal.binned
         if self._axes_manager != signal.axes_manager:
             self._axes_manager = signal.axes_manager
             self._create_arrays()
 
     def as_dictionary(self, fullcopy=True):
-        """Returns component as a dictionary. For more information on method
+        """
+        Returns component as a dictionary. For more information on method
         and conventions, see
-        py:meth:`~hyperspy.misc.export_dictionary.export_to_dictionary`
+        :py:meth:`~hyperspy.misc.export_dictionary.export_to_dictionary`
 
         Parameters
         ----------
@@ -1179,7 +1236,8 @@ class Component(t.HasTraits):
         return dic
 
     def _load_dictionary(self, dic):
-        """Load data from dictionary.
+        """
+        Load data from dictionary.
 
         Parameters
         ----------
@@ -1207,7 +1265,7 @@ class Component(t.HasTraits):
 
         if dic['_id_name'] == self._id_name:
             if (self._id_name == "Polynomial" and
-                    LooseVersion(hyperspy.__version__) >= LooseVersion("2.0")):
+                    Version(hyperspy.__version__) >= Version("2.0")):
                 # in HyperSpy 2.0 the polynomial definition changed
                 from hyperspy._components.polynomial import convert_to_polynomial
                 dic = convert_to_polynomial(dic)
@@ -1228,7 +1286,9 @@ class Component(t.HasTraits):
                     \ndictionary['_id_name'] = %s" % (self._id_name, dic['_id_name']))
 
     def print_current_values(self, only_free=False, fancy=True):
-        """Prints the current values of the component's parameters.
+        """
+        Prints the current values of the component's parameters.
+
         Parameters
         ----------
         only_free : bool
@@ -1239,5 +1299,71 @@ class Component(t.HasTraits):
         if fancy:
             display(current_component_values(self, only_free=only_free))
         else:
-            display_pretty(current_component_values(
-                self, only_free=only_free))
+            display_pretty(current_component_values(self, only_free=only_free))
+
+    @property
+    def _constant_term(self):
+        """
+        Get value of any (non-free) constant term of the component.
+        Returns 0 for most components.
+        """
+        return 0
+
+    def _compute_constant_term(self):
+        """Gets the value of any (non-free) constant term, with convolution"""
+        model = self.model
+        if model.convolved and self.convolved:
+            data = convolve_component_values(self._constant_term, model=model)
+        else:
+            signal_shape = model.axes_manager.signal_shape[::-1]
+            data = self._constant_term * np.ones(signal_shape)
+        return data.T[np.where(model.channel_switches)[::-1]].T
+
+
+def convolve_component_values(component_values, model):
+    """
+    Convolve component with model convolution axis.
+
+    Multiply by np.ones in order to handle case where component_values is a
+    single constant
+    """
+
+    sig = component_values * np.ones(model.convolution_axis.shape)
+
+    ll = model.low_loss(model.axes_manager)
+    convolved = np.convolve(sig, ll, mode="valid")
+
+    return convolved
+
+
+def _get_scaling_factor(signal, axis, parameter):
+    """
+    Convenience function to get the scaling factor required to take into
+    account binned and/or non-uniform axes.
+
+    Parameters
+    ----------
+    signal : BaseSignal
+    axis : BaseDataAxis
+    parameter : float or numpy array
+        The axis value at which scaling factor is evaluated (ignored if the axis
+        is uniform)
+
+    Returns
+    -------
+    scaling_factor
+
+    """
+
+    if is_binned(signal):
+    # in v2 replace by
+    #if axis.is_binned:
+        if axis.is_uniform:
+            scaling_factor = axis.scale
+        else:
+            parameter_idx  = axis.value2index(parameter)
+            scaling_factor = np.gradient(axis.axis)[parameter_idx]
+    else:
+        scaling_factor = 1
+
+    return scaling_factor
